package com.example.PhidgetFrequencyCounterExample;

import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.phidget22.*;

import java.text.DecimalFormat;

public class FrequencyCounterExample extends Activity {

	FrequencyCounter ch;
	CheckBox enabledBox;
	SeekBar dataIntervalBar;
	Spinner filterTypeSpinner;
	Spinner powerSupplySpinner;
	
	Toast errToast;

	int minDataInterval;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //Hide device information and settings until one is attached
		LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
		settingsAndData.setVisibility(LinearLayout.GONE);

		//set up enabled functionality
		enabledBox = (CheckBox) findViewById(R.id.enabledBox);
		enabledBox.setOnCheckedChangeListener(new enabledChangeListener());
		LinearLayout enabledSection = (LinearLayout) findViewById(R.id.enabledSection);
		enabledSection.setVisibility(LinearLayout.GONE);

		//set data interval seek bar functionality
		dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
		dataIntervalBar.setOnSeekBarChangeListener(new dataIntervalChangeListener());

		//set filter type spinner functionality
		filterTypeSpinner = (Spinner) findViewById(R.id.filterTypeSpinner);
		filterTypeSpinner.setOnItemSelectedListener(new filterTypeChangeListener());
		LinearLayout filterTypeSection = (LinearLayout) findViewById(R.id.filterTypeSection);
		filterTypeSection.setVisibility(LinearLayout.GONE);

		//set power supply spinner functionality
		powerSupplySpinner = (Spinner) findViewById(R.id.powerSupplySpinner);
		powerSupplySpinner.setOnItemSelectedListener(new powerSupplyChangeListener());
		LinearLayout powerSupplySection = (LinearLayout) findViewById(R.id.powerSupplySection);
		powerSupplySection.setVisibility(LinearLayout.GONE);
		
        try
        {
        	ch = new FrequencyCounter();

        	//Allow direct USB connection of Phidgets
			if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
                com.phidget22.usb.Manager.Initialize(this);

			//Enable server discovery to list remote Phidgets
			this.getSystemService(Context.NSD_SERVICE);
			Net.enableServerDiscovery(ServerType.DEVICE_REMOTE);

			ch.addAttachListener(new AttachListener() {
				public void onAttach(final AttachEvent attachEvent) {
				    AttachEventHandler handler = new AttachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addDetachListener(new DetachListener() {
				public void onDetach(final DetachEvent detachEvent) {
                    DetachEventHandler handler = new DetachEventHandler(ch);
                    synchronized(handler)
					{
						runOnUiThread(handler);
						try {
							handler.wait();
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			});

			ch.addErrorListener(new ErrorListener() {
				public void onError(final ErrorEvent errorEvent) {
					ErrorEventHandler handler = new ErrorEventHandler(ch, errorEvent);
					runOnUiThread(handler);

				}
			});

			ch.addFrequencyChangeListener(new FrequencyCounterFrequencyChangeListener() {
				public void onFrequencyChange(FrequencyCounterFrequencyChangeEvent frequencyChangeEvent) {
                    FrequencyCounterFrequencyChangeEventHandler handler = new FrequencyCounterFrequencyChangeEventHandler(ch, frequencyChangeEvent);
                    runOnUiThread(handler);
                }
			});

			ch.addCountChangeListener(new FrequencyCounterCountChangeListener() {
				public void onCountChange(FrequencyCounterCountChangeEvent countChangeEvent) {
					FrequencyCounterCountChangeEventHandler handler = new FrequencyCounterCountChangeEventHandler(ch, countChangeEvent);
					runOnUiThread(handler);
				}
			});

			ch.open();
        } catch (PhidgetException pe) {
	        pe.printStackTrace();
		}

    }

	private class enabledChangeListener implements CheckBox.OnCheckedChangeListener {
		@Override
		public void onCheckedChanged(CompoundButton buttonView, boolean isChecked)
		{
			try {
				ch.setEnabled(isChecked);
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}
	}

	private class dataIntervalChangeListener implements SeekBar.OnSeekBarChangeListener {
		public void onProgressChanged(SeekBar seekBar, int progress,
									  boolean fromUser) {
			try {
				TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
				int dataInterval = progress + minDataInterval;
				dataIntervalTxt.setText(String.valueOf(dataInterval));
				ch.setDataInterval(dataInterval);
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		public void onStartTrackingTouch(SeekBar seekBar) {}

		public void onStopTrackingTouch(SeekBar seekBar) {}
	}

	private class filterTypeChangeListener implements Spinner.OnItemSelectedListener {
		@Override
		public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
			try {
				ch.setFilterType(FrequencyFilterType.valueOf(parentView.getItemAtPosition(position).toString()));
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		@Override
		public void onNothingSelected(AdapterView<?> parentView) {
			
		}

	}

	private class powerSupplyChangeListener implements Spinner.OnItemSelectedListener {
		@Override
		public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
			try {
				ch.setPowerSupply(PowerSupply.valueOf(parentView.getItemAtPosition(position).toString()));
			} catch (PhidgetException e) {
				e.printStackTrace();
			}
		}

		@Override
		public void onNothingSelected(AdapterView<?> parentView) {
			
		}

	}

    class AttachEventHandler implements Runnable { 
    	Phidget ch;

		public AttachEventHandler(Phidget ch) {
			this.ch = ch;
		}

		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);
			settingsAndData.setVisibility(LinearLayout.VISIBLE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);

			attachedTxt.setText("Attached");
			try {
				TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
				TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
				TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
				TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
				TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
				TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

				nameTxt.setText(ch.getDeviceName());
				serialTxt.setText(Integer.toString(ch.getDeviceSerialNumber()));
				versionTxt.setText(Integer.toString(ch.getDeviceVersion()));
				channelTxt.setText(Integer.toString(ch.getChannel()));
				hubPortTxt.setText(Integer.toString(ch.getHubPort()));
				labelTxt.setText(ch.getDeviceLabel());

				TextView dataIntervalTxt = (TextView) findViewById(R.id.dataIntervalTxt);
				dataIntervalTxt.setText(String.valueOf(((FrequencyCounter)ch).getDataInterval()));

				minDataInterval = ((FrequencyCounter)ch).getMinDataInterval();

				SeekBar dataIntervalBar = (SeekBar) findViewById(R.id.dataIntervalBar);
				dataIntervalBar.setProgress(((FrequencyCounter)ch).getDataInterval() - minDataInterval);

				//Limit the maximum dataInterval on the SeekBar to 5000 so it remains usable
				if(((FrequencyCounter)ch).getMaxDataInterval() >= 5000)
                    dataIntervalBar.setMax(5000 - minDataInterval);
				else
                    dataIntervalBar.setMax(((FrequencyCounter)ch).getMaxDataInterval() - minDataInterval);

				enabledBox.setChecked(((FrequencyCounter)ch).getEnabled());

				switch(ch.getDeviceID()) {
					case PN_1054:
						filterTypeSpinner.setAdapter(new ArrayAdapter<FrequencyFilterType>(getApplicationContext(),
								android.R.layout.simple_spinner_item, FrequencyFilterType.values()));

						//Logic Level by default
						filterTypeSpinner.setSelection(1);

						((LinearLayout)findViewById(R.id.filterTypeSection)).setVisibility(LinearLayout.VISIBLE);
						((LinearLayout)findViewById(R.id.enabledSection)).setVisibility(LinearLayout.VISIBLE);
						break;
					case PN_DAQ1400:
						powerSupplySpinner.setAdapter(new ArrayAdapter<PowerSupply>(getApplicationContext(),
								android.R.layout.simple_spinner_item, PowerSupply.values()));

						//Off by default
						powerSupplySpinner.setSelection(0);

						((LinearLayout)findViewById(R.id.powerSupplySection)).setVisibility(LinearLayout.VISIBLE);
						break;
					default:
						break;
				}
			} catch (PhidgetException e) {
				e.printStackTrace();
			}

			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }
    
    class DetachEventHandler implements Runnable {
    	Phidget ch;
    	
    	public DetachEventHandler(Phidget ch) {
    		this.ch = ch;
    	}
    	
		public void run() {
			LinearLayout settingsAndData = (LinearLayout) findViewById(R.id.settingsAndData);

			settingsAndData.setVisibility(LinearLayout.GONE);

			TextView attachedTxt = (TextView) findViewById(R.id.attachedTxt);
			attachedTxt.setText("Detached");

			TextView nameTxt = (TextView) findViewById(R.id.nameTxt);
			TextView serialTxt = (TextView) findViewById(R.id.serialTxt);
			TextView versionTxt = (TextView) findViewById(R.id.versionTxt);
			TextView channelTxt = (TextView) findViewById(R.id.channelTxt);
			TextView hubPortTxt = (TextView) findViewById(R.id.hubPortTxt);
			TextView labelTxt = (TextView) findViewById(R.id.labelTxt);

			nameTxt.setText(R.string.unknown_val);
			serialTxt.setText(R.string.unknown_val);
			versionTxt.setText(R.string.unknown_val);
			channelTxt.setText(R.string.unknown_val);
			hubPortTxt.setText(R.string.unknown_val);
			labelTxt.setText(R.string.unknown_val);

			((LinearLayout)findViewById(R.id.filterTypeSection)).setVisibility(LinearLayout.GONE);
			((LinearLayout)findViewById(R.id.enabledSection)).setVisibility(LinearLayout.GONE);
			((LinearLayout)findViewById(R.id.powerSupplySection)).setVisibility(LinearLayout.GONE);

			((TextView)findViewById(R.id.frequencyTxt)).setText("");
			((TextView)findViewById(R.id.countTxt)).setText("");
			((TextView)findViewById(R.id.timeChangeTxt)).setText("");
			
			//notify that we're done
			synchronized(this)
			{
				this.notify();
			}
		}
    }

	class ErrorEventHandler implements Runnable {
		Phidget ch;
		ErrorEvent errorEvent;

		public ErrorEventHandler(Phidget ch, ErrorEvent errorEvent) {
			this.ch = ch;
			this.errorEvent = errorEvent;
		}

		public void run() {
			 if (errToast == null)
				 errToast = Toast.makeText(getApplicationContext(), errorEvent.getDescription(), Toast.LENGTH_SHORT);

			 //replace the previous toast message if a new error occurs
			 errToast.setText(errorEvent.getDescription());
			 errToast.show();
        }
	}

	class FrequencyCounterFrequencyChangeEventHandler implements Runnable {
		Phidget ch;
		FrequencyCounterFrequencyChangeEvent frequencyChangeEvent;

		public FrequencyCounterFrequencyChangeEventHandler(Phidget ch, FrequencyCounterFrequencyChangeEvent frequencyChangeEvent) {
			this.ch = ch;
			this.frequencyChangeEvent = frequencyChangeEvent;
		}

		public void run() {
		    TextView frequencyTxt = (TextView)findViewById(R.id.frequencyTxt);

			frequencyTxt.setText(String.valueOf(frequencyChangeEvent.getFrequency()));
		}
	}

	class FrequencyCounterCountChangeEventHandler implements Runnable {
		Phidget ch;
		FrequencyCounterCountChangeEvent countChangeEvent;

		public FrequencyCounterCountChangeEventHandler(Phidget ch, FrequencyCounterCountChangeEvent countChangeEvent) {
			this.ch = ch;
			this.countChangeEvent = countChangeEvent;
		}

		public void run() {
			TextView countTxt = (TextView)findViewById(R.id.countTxt);
			TextView timeChangeTxt = (TextView)findViewById(R.id.timeChangeTxt);

			DecimalFormat numberFormat = new DecimalFormat("#.##");

			countTxt.setText(String.valueOf(countChangeEvent.getCounts()));
			timeChangeTxt.setText(numberFormat.format(countChangeEvent.getTimeChange()));
		}
	}

    @Override
    protected void onDestroy() {
    	super.onDestroy();
    	try {
			ch.close();

		} catch (PhidgetException e) {
			e.printStackTrace();
		}

		//Disable USB connection to Phidgets
    	if(getPackageManager().hasSystemFeature(PackageManager.FEATURE_USB_HOST))
            com.phidget22.usb.Manager.Uninitialize();
    }

}

